@@ -10,7 +10,7 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <tk.h>
+#include <tkInt.h>
 #include <X11/Xatom.h>
 
 #include "ttkTheme.h"
@@ -652,7 +652,7 @@ static void EntryRevalidateBG(Entry *entryPtr, VREASON reason)
 {
     Tcl_Interp *interp = entryPtr->core.interp;
     if (EntryRevalidate(interp, entryPtr, reason) == TCL_ERROR) {
-	Tcl_BackgroundError(interp);
+	Tcl_BackgroundException(interp, TCL_ERROR);
     }
 }
 
@@ -758,8 +758,8 @@ static int EntrySetValue(Entry *entryPtr, const char *value)
 	    Tcl_GetString(entryPtr->entry.textVariableObj);
 	if (textVarName && *textVarName) {
 	    entryPtr->core.flags |= SYNCING_VARIABLE;
-	    value = Tcl_SetVar(entryPtr->core.interp, textVarName,
-		    value, TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
+	    value = Tcl_SetVar2(entryPtr->core.interp, textVarName,
+		    NULL, value, TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
 	    entryPtr->core.flags &= ~SYNCING_VARIABLE;
 	    if (!value || WidgetDestroyed(&entryPtr->core)) {
 		return TCL_ERROR;
@@ -786,7 +786,7 @@ static void EntryTextVariableTrace(void *recordPtr, const char *value)
     }
 
     if (entryPtr->core.flags & SYNCING_VARIABLE) {
-	/* Trace was fired due to Tcl_SetVar call in EntrySetValue.
+	/* Trace was fired due to Tcl_SetVar2 call in EntrySetValue.
 	 * Don't do anything.
 	 */
 	return;
@@ -1135,13 +1135,14 @@ EntryDoLayout(void *recordPtr)
  *      Get a GC using the specified foreground color and the entry's font.
  *      Result must be freed with Tk_FreeGC().
  */
-static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj)
+static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj, TkRegion clip)
 {
     Tk_Window tkwin = entryPtr->core.tkwin;
     Tk_Font font = Tk_GetFontFromObj(tkwin, entryPtr->entry.fontObj);
     XColor *colorPtr;
     unsigned long mask = 0ul;
     XGCValues gcValues;
+    GC gc;
 
     gcValues.line_width = 1; mask |= GCLineWidth;
     gcValues.font = Tk_FontId(font); mask |= GCFont;
@@ -1149,7 +1150,11 @@ static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj)
 	gcValues.foreground = colorPtr->pixel;
 	mask |= GCForeground;
     }
-    return Tk_GetGC(entryPtr->core.tkwin, mask, &gcValues);
+    gc = Tk_GetGC(entryPtr->core.tkwin, mask, &gcValues);
+    if (clip != None) {
+	TkSetRegion(Tk_Display(entryPtr->core.tkwin), gc, clip);
+    }
+    return gc;
 }
 
 /* EntryDisplay --
@@ -1160,23 +1165,27 @@ static void EntryDisplay(void *clientData, Drawable d)
     Entry *entryPtr = clientData;
     Tk_Window tkwin = entryPtr->core.tkwin;
     int leftIndex = entryPtr->entry.xscroll.first,
-	rightIndex = entryPtr->entry.xscroll.last,
+	rightIndex = entryPtr->entry.xscroll.last + 1,
 	selFirst = entryPtr->entry.selectFirst,
 	selLast = entryPtr->entry.selectLast;
     EntryStyleData es;
     GC gc;
     int showSelection, showCursor;
+    Ttk_Box textarea;
+    TkRegion clipRegion;
+    XRectangle rect;
 
     EntryInitStyleData(entryPtr, &es);
 
+    textarea = Ttk_ClientRegion(entryPtr->core.layout, "textarea");
     showCursor =
-	   (entryPtr->core.flags & CURSOR_ON) != 0
+	   (entryPtr->core.flags & CURSOR_ON)
 	&& EntryEditable(entryPtr)
 	&& entryPtr->entry.insertPos >= leftIndex
 	&& entryPtr->entry.insertPos <= rightIndex
 	;
     showSelection =
-	   (entryPtr->core.state & TTK_STATE_DISABLED) == 0
+	   !(entryPtr->core.state & TTK_STATE_DISABLED)
 	&& selFirst > -1
 	&& selLast > leftIndex
 	&& selFirst <= rightIndex
@@ -1214,6 +1223,20 @@ static void EntryDisplay(void *clientData, Drawable d)
 	}
     }
 
+    /* Initialize the clip region. Note that Xft does _not_ derive its
+     * clipping area from the GC, so we have to supply that by other means.
+     */
+
+    rect.x = textarea.x;
+    rect.y = textarea.y;
+    rect.width = textarea.width;
+    rect.height = textarea.height;
+    clipRegion = TkCreateRegion();
+    TkUnionRectWithRegion(&rect, clipRegion, clipRegion);
+#ifdef HAVE_XFT
+    TkUnixSetXftClipRegion(clipRegion);
+#endif
+
     /* Draw cursor:
      */
     if (showCursor) {
@@ -1230,31 +1253,42 @@ static void EntryDisplay(void *clientData, Drawable d)
 	/* @@@ should: maybe: SetCaretPos even when blinked off */
 	Tk_SetCaretPos(tkwin, cursorX, cursorY, cursorHeight);
 
-	gc = EntryGetGC(entryPtr, es.insertColorObj);
+	gc = EntryGetGC(entryPtr, es.insertColorObj, clipRegion);
 	XFillRectangle(Tk_Display(tkwin), d, gc,
 	    cursorX-cursorWidth/2, cursorY, cursorWidth, cursorHeight);
+	XSetClipMask(Tk_Display(tkwin), gc, None);
 	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
 
     /* Draw the text:
      */
-    gc = EntryGetGC(entryPtr, es.foregroundObj);
+    gc = EntryGetGC(entryPtr, es.foregroundObj, clipRegion);
     Tk_DrawTextLayout(
 	Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
 	entryPtr->entry.layoutX, entryPtr->entry.layoutY,
 	leftIndex, rightIndex);
+    XSetClipMask(Tk_Display(tkwin), gc, None);
     Tk_FreeGC(Tk_Display(tkwin), gc);
 
     /* Overwrite the selected portion (if any) in the -selectforeground color:
      */
     if (showSelection) {
-	gc = EntryGetGC(entryPtr, es.selForegroundObj);
+	gc = EntryGetGC(entryPtr, es.selForegroundObj, clipRegion);
 	Tk_DrawTextLayout(
 	    Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
 	    entryPtr->entry.layoutX, entryPtr->entry.layoutY,
 	    selFirst, selLast);
+	XSetClipMask(Tk_Display(tkwin), gc, None);
 	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
+
+    /* Drop the region. Note that we have to manually remove the reference to
+     * it from the Xft guts (if they're being used).
+     */
+#ifdef HAVE_XFT
+    TkUnixSetXftClipRegion(None);
+#endif
+    TkDestroyRegion(clipRegion);
 }
 
 /*------------------------------------------------------------------------
@@ -1280,8 +1314,8 @@ EntryIndex(
     int *indexPtr)		/* Return value */
 {
 #   define EntryWidth(e) (Tk_Width(entryPtr->core.tkwin)) /* Not Right */
-    int length;
-    const char *string = Tcl_GetStringFromObj(indexObj, &length);
+    const char *string = Tcl_GetString(indexObj);
+    size_t length = indexObj->length;
 
     if (strncmp(string, "end", length) == 0) {
 	*indexPtr = entryPtr->entry.numChars;
@@ -1293,9 +1327,10 @@ EntryIndex(
 	*indexPtr = entryPtr->entry.xscroll.last;
     } else if (strncmp(string, "sel.", 4) == 0) {
 	if (entryPtr->entry.selectFirst < 0) {
-	    Tcl_ResetResult(interp);
-	    Tcl_AppendResult(interp, "selection isn't in widget ",
-		    Tk_PathName(entryPtr->core.tkwin), NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "selection isn't in widget %s",
+		    Tk_PathName(entryPtr->core.tkwin)));
+	    Tcl_SetErrorCode(interp, "TTK", "ENTRY", "NO_SELECTION", NULL);
 	    return TCL_ERROR;
 	}
 	if (strncmp(string, "sel.first", length) == 0) {
@@ -1347,8 +1382,9 @@ EntryIndex(
     return TCL_OK;
 
 badIndex:
-    Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp, "bad entry index \"", string, "\"", NULL);
+    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+	    "bad entry index \"%s\"", string));
+    Tcl_SetErrorCode(interp, "TTK", "ENTRY", "INDEX", NULL);
     return TCL_ERROR;
 }
 
@@ -1423,7 +1459,7 @@ EntryGetCommand(
 	Tcl_WrongNumArgs(interp, 2, objv, NULL);
 	return TCL_ERROR;
     }
-    Tcl_SetResult(interp, entryPtr->entry.string, TCL_VOLATILE);
+    Tcl_SetObjResult(interp, Tcl_NewStringObj(entryPtr->entry.string, -1));
     return TCL_OK;
 }
 
@@ -1608,6 +1644,14 @@ static int EntryXViewCommand(
     void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
     Entry *entryPtr = recordPtr;
+    if (objc == 3) {
+	int newFirst;
+	if (EntryIndex(interp, entryPtr, objv[2], &newFirst) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	TtkScrollTo(entryPtr->entry.xscrollHandle, newFirst);
+	return TCL_OK;
+    }
     return TtkScrollviewCommand(interp, objc, objv, entryPtr->entry.xscrollHandle);
 }
 
@@ -1753,9 +1797,9 @@ static int ComboboxCurrentCommand(
 	    return TCL_ERROR;
 	}
 	if (currentIndex < 0 || currentIndex >= nValues) {
-	    Tcl_AppendResult(interp,
-		    "Index ", Tcl_GetString(objv[2]), " out of range",
-		    NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "Index %s out of range", Tcl_GetString(objv[2])));
+	    Tcl_SetErrorCode(interp, "TTK", "COMBOBOX", "IDX_RANGE", NULL);
 	    return TCL_ERROR;
 	}
 
